/**
 * 电子围栏类
 */


const Sequelize = require('sequelize');
let xcSequelize = null;
let xcRedis = null;

const CREATE_RULE = {
    freezeTableName: true,
    paranoid: true
}; //添加删除deletedAt

/**
 * 围栏类型
 */
const GFENCE_TYPE = {
    atService: 'atService', //服务区
    usable: 'usable', //租停区
    denied: 'denied', //禁行区
    //other
}

/**
 * 围栏形状
 */
const SHAPE_TYPE = {
    polygon: 'polygon',
    circle: 'circle'
};

const XC_EBIKE_GFENCE_DS = {
    id: {
        type: Sequelize.INTEGER(8).UNSIGNED,
        primaryKey: true,
        allowNull: false,
        autoIncrement: true
    },
    name: { //添加用户新建时的区域名字
        type: Sequelize.STRING(128),
        unique: true,
        allowNull: false
    },
    type: {//电子围栏类型
        type: Sequelize.STRING(16),
        allowNull: false,
    },
    shapeType: {//电子围栏形状
        type: Sequelize.STRING(16),
        allowNull: false
    },
    centerLat: {
        type: Sequelize.DOUBLE(9, 6),
        allowNull: false
    },
    centerLng: {
        type: Sequelize.DOUBLE(9, 6),
        allowNull: false,
    },
    pointList: { //电子围栏顶点列表。json串
        type: Sequelize.STRING(1024),
        allowNull: false
    }

}

class XCEBikeGFence {

    /**
     * 配置redis client
     */
    static configRedis(redisClient, redisSubClient) {
        xcRedis = redisClient;
    }

    static get SHAPE_TYPE() {
        return SHAPE_TYPE;
    }

    static get GFENCE_TYPE(){
        return GFENCE_TYPE;
    }
    /**
     * 配置数据库实例
     */
    static configSequelize(sequelizeInstance) {
        xcSequelize = sequelizeInstance;
    }

    static log() {
        console.log(xcRedis, xcSequelize);
    }


    constructor(json) {
        //电子围栏类型
        this.type = json.type;
        //电子围栏所属的agentId
        this.agentId = json.agentId;
        //电子围栏id
        this.gFenceId = json.gFenceId;
        //电子围栏name
        this.name = json.name;
        //电子围栏形状
        this.shapeType = json.shapeType;
        //电子围栏中心点
        this.centerLng = json.centerLng;
        this.centerLat = json.centerLat;
        //电子围栏点列表{lat:lat,lng:lng}
        this.pointList = json.pointList;

    }

    /**
     * 电子围栏信息转mapData
     */
    mapData() {
        let map = new Map();
        map.set('gFenceId', this.gFenceId);
        map.set('type', this.type);
        map.set('name', this.name);
        map.set('shapeType', this.shapeType);
        map.set('centerLng', this.centerLng);
        map.set('centerLat', this.centerLat);
        map.set('pointList', this.pointList);
        map.set('agentId', this.agentId);

        return map;
    }

    /**
     * 组合地理围栏的Redis Key
     * @param {*} agentId 
     * @param {*} type 
     * @param {*} gFenceId 
     */
    static gFenceKey(agentId, gFenceId) {
        return `gFence_${agentId}_${gFenceId}`;
    }

    /**
     * 从Redis中初始化一个电子围栏
     * 没有查询到电子围栏信息则返回null
     */
    static async instanceFromRedis(agentId, gFenceId) {
        let json = await xcRedis.hgetall(XCEBikeGFence.gFenceKey(agentId, gFenceId));
        if (json && json.gFenceId) {
            let gFence = new XCEBikeGFence(json);
            return gFence;
        } else {
            return;
        }

    }

    /**
     * 将电子围栏保存在Redis中
     */
    async save2Redis() {
        let mapData = this.mapData();
        let ret = await xcRedis.hmset(XCEBikeGFence.gFenceKey(this.agentId, this.gFenceId), mapData);
        return ret;
    }

    /**
     * 地理围栏DAO
     * @param {*} agentId 
     */
    async gFenceDAO(agentId) {
        return xcSequelize.define(`xc_ebike_gfence_${agentId}`, XC_EBIKE_GFENCE_DS, CREATE_RULE);
    }

    /**
     * 创建租还区数据
     * @param {*} agentId 
     */
    async gFenceUsableCreate(agentId) {
        let usableDAO = await this.gFenceUsableDAO(agentId);
        await usableDAO.sync();
        let usableCreate = await usableDAO.upsert({
            name: this.name,
            shapeType: this.shapeType,
            centerLat: this.centerLat,
            centerLng: this.centerLng,
            pointList: this.pointList,
            deletedAt: null
        });
        let findOne = await usableDAO.findAll({
            where: {
                name: {
                    $eq: this.name
                }
            }
        });
        return findOne;
    }
    /**
     * 创建服务区数据
     * @param {*} agentId 
     */
    async gFenceAtServiceCreate(agentId) {
        let atServcieDAO = await this.gFenceAtServiceDAO(agentId);
        await atServcieDAO.sync();
        let atServiceCreate = await atServcieDAO.upsert({
            name: this.name,
            shapeType: this.shapeType,
            centerLat: this.centerLat,
            centerLng: this.centerLng,
            pointList: this.pointList,
            deletedAt: null
        });
        let findOne = await atServcieDAO.findAll({
            where: {
                name: {
                    $eq: this.name
                }
            }
        });
        return findOne;
    }

    /**
     * 获取租还区数据
     * @param {*} agentId 
     */
    async gFenceUsableGet(agentId) {
        let usableDAO = await this.gFenceUsableDAO(agentId);
        await usableDAO.sync();
        let usableFind = await usableDAO.findAll({
            where: {
                deletedAt: {
                    $eq: null
                }
            }
        });
        return usableFind;
    }
    /**
     * 获取服务区数据
     * @param {*} agentId 
     */
    async gFenceAtServiceGet(agentId) {
        let atServcieDAO = await this.gFenceAtServiceDAO(agentId);
        await atServcieDAO.sync();
        let atServiceFind = await atServcieDAO.findAll({
            where: {
                deletedAt: {
                    $eq: null
                }
            }
        });
        return atServiceFind;
    }

    /**
     * 更新租还区数据
     * @param {*} agentId 
     * @param {*} id 
     */
    async gFenceUsableUpdate(agentId, id) {
        let usableDAO = await this.gFenceUsableDAO(agentId);
        await usableDAO.sync();
        let usableUpdate = await usableDAO.update({
            name: this.name,
            shapeType: this.shapeType,
            centerLat: this.centerLat,
            centerLng: this.centerLng,
            pointList: this.pointList,
            deletedAt: null
        }, {
            where: {
                id: {
                    $eq: Number(id)
                }
            }
        });
        return usableUpdate;
    }
    /**
     * 更新服务区数据
     * @param {*} agentId 
     * @param {*} id 
     */
    async gFenceAtServiceUpdate(agentId, id) {
        let atServcieDAO = await this.gFenceAtServiceDAO(agentId);
        await atServcieDAO.sync();
        let atServiceUpdate = await atServcieDAO.update({
            name: this.name,
            shapeType: this.shapeType,
            centerLat: this.centerLat,
            centerLng: this.centerLng,
            pointList: this.pointList,
            deletedAt: null
        }, {
            where: {
                id: {
                    $eq: Number(id)
                }
            }
        });
        return atServiceUpdate;
    }
    /**
     * 删除id对应的租还区数据
     * @param {*} agentId 
     * @param {*} id 
     */
    async gFenceUsableDestroy(agentId, id) {
        let usableDAO = await this.gFenceUsableDAO(agentId);
        await usableDAO.sync();
        let usableDestroy = await usableDAO.destroy({
            where: {
                id: {
                    $eq: Number(id)
                }
            }
        });
        return usableDestroy;
    }
    /**
     * 删除id对应的服务区的数据
     * @param {*} agentId 
     * @param {*} id 
     */
    async gFenceAtServiceDestory(agentId, id) {
        let atServcieDAO = await this.gFenceAtServiceDAO(agentId);
        await atServcieDAO.sync();
        let atServiceDestroy = await atServcieDAO.destroy({
            where: {
                id: {
                    $eq: Number(id)
                }
            }
        });
        return atServiceDestroy;
    }

}


module.exports = XCEBikeGFence;